@@ -25,10 +25,10 @@
 
 typedef struct _GtkHTMLReplaceDialog GtkHTMLReplaceDialog;
 
-#include <gnome.h>
+#include "gtkhtml.h"
 #include "control-data.h"
 
-GtkHTMLReplaceDialog * gtk_html_replace_dialog_new     (GtkHTML *html);
+GtkHTMLReplaceDialog * gtk_html_replace_dialog_new     (GtkHTML *html, GtkHTMLControlData *cd);
 void                   gtk_html_replace_dialog_destroy (GtkHTMLReplaceDialog *d);
 
 void                   replace                         (GtkHTMLControlData *cd);
