@@ -108,6 +108,8 @@ destroy (HTMLObject *o)
 
 	if (table->bgColor)
 		gdk_color_free (table->bgColor);
+	if (table->bgPixmap)
+		html_image_factory_unregister (table->bgPixmap->factory, table->bgPixmap, NULL);
 
 	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
@@ -124,6 +126,7 @@ copy_sized (HTMLObject *self, HTMLObject *dest, gint rows, gint cols)
 
 	d->bgColor     = s->bgColor ? gdk_color_copy (s->bgColor) : NULL;
 	d->caption     = s->caption ? HTML_CLUEV (html_object_dup (HTML_OBJECT (s->caption))) : NULL;
+	d->bgPixmap    = s->bgPixmap ? html_image_factory_register(s->bgPixmap->factory, NULL, s->bgPixmap->url, FALSE) : NULL;
 
 	d->columnMin   = g_array_new (FALSE, FALSE, sizeof (gint));
 	d->columnFixed = g_array_new (FALSE, FALSE, sizeof (gint));
@@ -1257,12 +1260,12 @@ draw (HTMLObject *o,
 		if (table->caption && table->capAlign == HTML_VALIGN_TOP)
 			g_print ("FIXME: Support captions\n");
 
-		html_painter_draw_panel (p, html_object_get_bg_color (o->parent, p),
-					 tx, ty + capOffset, 
-					 HTML_OBJECT (table)->width,
-					 ROW_HEIGHT (table, table->totalRows) +
-					 pixel_size * table->border, GTK_HTML_ETCH_OUT,
-					 pixel_size * table->border);
+		html_painter_draw_border (p, html_object_get_bg_color (o->parent, p),
+					  tx, ty + capOffset, 
+					  HTML_OBJECT (table)->width,
+					  ROW_HEIGHT (table, table->totalRows) +
+					  pixel_size * table->border, HTML_BORDER_OUTSET,
+					  pixel_size * table->border);
 
 		/* Draw borders around each cell */
 		for (r = start_row; r <= end_row; r++) {
@@ -1276,16 +1279,16 @@ draw (HTMLObject *o,
 				    table->cells[r + 1][c] == cell)
 					continue;
 
-				html_painter_draw_panel (p, html_object_get_bg_color (HTML_OBJECT (cell), p),
-							 tx + COLUMN_OPT (table, cell->col),
-							 ty + ROW_HEIGHT (table, cell->row) + capOffset,
-							 (COLUMN_OPT (table, c + 1)
-							  - COLUMN_OPT (table, cell->col)
-							  - pixel_size * table->spacing),
-							 (ROW_HEIGHT (table, r + 1)
-							  - ROW_HEIGHT (table, cell->row)
-							  - pixel_size * table->spacing),
-							 GTK_HTML_ETCH_IN, pixel_size);
+				html_painter_draw_border (p, html_object_get_bg_color (HTML_OBJECT (cell), p),
+							  tx + COLUMN_OPT (table, cell->col),
+							  ty + ROW_HEIGHT (table, cell->row) + capOffset,
+							  (COLUMN_OPT (table, c + 1)
+							   - COLUMN_OPT (table, cell->col)
+							   - pixel_size * table->spacing),
+							  (ROW_HEIGHT (table, r + 1)
+							   - ROW_HEIGHT (table, cell->row)
+							   - pixel_size * table->spacing),
+							  HTML_BORDER_INSET, pixel_size);
 						      
 			}
 		}
@@ -1469,11 +1472,12 @@ divide_into_percented (HTMLTable *table, gint *col_percent, gint *max_size, gint
 
 static gboolean
 calc_lowest_fill (HTMLTable *table, GArray *pref, gint *max_size, gint *col_percent, gint pixel_size,
-		  gint *ret_col, gint *ret_total_pref)
+		  gint *ret_col, gint *ret_total_pref, gint *ret_total)
 {
 	gint c, pw, border_extra = table->border ? 2 : 0, min_fill = COLUMN_PREF (table, table->totalCols);
 
 	*ret_total_pref = 0;
+	*ret_total = 0;
 	for (c = 0; c < table->totalCols; c++)
 		if (col_percent [c + 1] == col_percent [c]) {
 			pw = PREF (c + 1) - PREF (c)
@@ -1486,6 +1490,7 @@ calc_lowest_fill (HTMLTable *table, GArray *pref, gint *max_size, gint *col_perc
 				}
 
 				(*ret_total_pref) += pw;
+				(*ret_total) += max_size [c];
 			}
 		}
 
@@ -1497,27 +1502,25 @@ divide_upto_preferred_width (HTMLTable *table, HTMLPainter *painter, GArray *pre
 			     gint *col_percent, gint *max_size, gint left)
 {
 	gint added, part, c, pw, pixel_size = html_painter_get_pixel_size (painter);
-	gint total_fill, min_col, min_fill, min_pw, processed_pw, border_extra = table->border ? 2 : 0;
+	gint total, total_pref, to_divide, min_col, min_fill, min_pw, processed_pw, border_extra = table->border ? 2 : 0;
 
-	while (left && calc_lowest_fill (table, pref, max_size, col_percent, pixel_size, &min_col, &total_fill)) {
+	/* printf ("cols: %d left: %d\n", table->totalCols, left); */
+	while (left > 0 && calc_lowest_fill (table, pref, max_size, col_percent, pixel_size, &min_col, &total_pref, &total)) {
 		min_pw   = PREF (min_col + 1) - PREF (min_col)
 			- pixel_size * (table->spacing + border_extra);
-		min_fill = MIN (min_pw - max_size [min_col], ((gdouble) min_pw * left) / total_fill);
-		if (min_fill <= 0)
-			break;
-		/* printf ("min_col %d min_pw %d MIN(%d,%d)\n", min_col, min_pw, min_pw - max_size [min_col],
-		   (gint)(((gdouble) min_pw * left) / total_fill)); */
-
-		if (min_fill == min_pw - max_size [min_col]) {
-			/* first add to minimal one */
-			max_size [min_col] += min_fill;
-			left -= min_fill;
-			total_fill -= min_pw;
-			/* printf ("min satisfied %d, (%d=%d)left: %d\n", min_fill, max_size [min_col], min_pw, left); */
+		/* printf ("min: %d left: %d\n", min_col, left); */
+		to_divide = MIN (total_pref - total, left);
+		if (min_pw - max_size [min_col] < ((gdouble) min_pw * to_divide) / total_pref) {
+			added = min_pw - max_size [min_col];
+			left -= added;
+			min_fill = to_divide - added;
+			max_size [min_col] += added;
+			total_pref -= min_pw;
+		} else {
+			min_fill = to_divide;
 		}
-		if (!left)
-			break;
 
+		/* printf ("min satisfied %d, (%d=%d) left: %d\n", min_fill, max_size [min_col], min_pw, left); */
 		processed_pw = 0;
 		added = 0;
 
@@ -1527,11 +1530,13 @@ divide_upto_preferred_width (HTMLTable *table, HTMLPainter *painter, GArray *pre
 					- pixel_size * (table->spacing + border_extra);
 				if (max_size [c] < pw) {
 					processed_pw += pw;
-					part = (LL min_fill * processed_pw) / total_fill;
-					if (LL min_fill * processed_pw - LL part * total_fill
-					    > LL (part + 1) * total_fill - LL min_fill * processed_pw)
+					part = (LL min_fill * processed_pw) / total_pref;
+					if (LL min_fill * processed_pw - LL part * total_pref
+					    > LL (part + 1) * total_pref - LL min_fill * processed_pw)
 						part ++;
 					part         -= added;
+					if (max_size [c] + part > pw)
+						part = pw - max_size [c];
 					added        += part;
 					max_size [c] += part;
 					left         -= part;
@@ -1541,6 +1546,8 @@ divide_upto_preferred_width (HTMLTable *table, HTMLPainter *painter, GArray *pre
 		}
 	}
 
+	/* printf ("cols: %d left: %d (END)\n", table->totalCols, left); */
+
 	return left;
 }
 
@@ -1873,18 +1880,20 @@ check_point (HTMLObject *self,
 			if (r < end_row - 1 && table->cells[r+1][c] == cell)
 				continue;
 
-			/* correct to include cell spacing */
 			co = HTML_OBJECT (cell);
 			cx = x;
 			cy = y;
-			if (x < co->x && x >= co->x - hsa - tbc)
-				cx = co->x;
-			if (x >= co->x + co->width && x < co->x + co->width + hsb + tbc)
-				cx = co->x + co->width - 1;
-			if (y < co->y - co->ascent && y >= co->y - co->ascent - hsa - tbc)
-				cy = co->y - co->ascent;
-			if (y >= co->y + co->descent && y < co->y + co->descent + hsb + tbc)
-				cy = co->y + co->descent - 1;
+			if (for_cursor) {
+				/* correct to include cell spacing */
+				if (x < co->x && x >= co->x - hsa - tbc)
+					cx = co->x;
+				if (x >= co->x + co->width && x < co->x + co->width + hsb + tbc)
+					cx = co->x + co->width - 1;
+				if (y < co->y - co->ascent && y >= co->y - co->ascent - hsa - tbc)
+					cy = co->y - co->ascent;
+				if (y >= co->y + co->descent && y < co->y + co->descent + hsb + tbc)
+					cy = co->y + co->descent - 1;
+			}
 
 			obj = html_object_check_point (HTML_OBJECT (cell), painter, cx, cy, offset_return, for_cursor);
 			if (obj != NULL)
@@ -1892,6 +1901,18 @@ check_point (HTMLObject *self,
 		}
 	}
 
+	if (!for_cursor) {
+		if (x >= 0 && y >= 0 && x < self->width && y < self->ascent + self->descent) {
+			if (offset_return) {
+				if (x < self->width/2)
+					*offset_return = 0;
+				else
+					*offset_return = 1;
+			}
+			return self;
+		}
+	}
+
 	return NULL;
 }
 
